@@ -1,4 +1,4 @@
-`MI.random.times` <-
+MI.random.times <-
 function (time.points) {
     # indexes for missing data
     t.max <- if (is.null(tt <- attr(time.points, "t.max"))) max(obs.times) else tt
@@ -10,6 +10,10 @@ function (time.points) {
     Z.missO <- Z
     Xtime.missO <- Xtime
     Ztime.missO <- Ztime
+    P.missO <- P
+    log.st.missO <- log.st
+    Xs.missO <- Xs
+    Zs.missO <- Zs
     WW.missO <- WW
     n.missO <- nrow(Ztime.missO)
     id.miss <- id3.miss <- id
@@ -18,10 +22,10 @@ function (time.points) {
     diag.D <- ncz != ncol(D)
     thets <- c(object$coefficients$gammas, object$coefficients$alpha)
     thetas <- c(object$coefficients$betas, log(object$coefficients$sigma),
-        if (object$method == "weibull-GH" || object$method == "ph-GH") thets else {
+        if (object$method %in% c("weibull-PH-GH", "weibull-AFT-GH", "ph-GH")) thets else {
             thets[2:nk] <- log(diff(thets[1:nk]))
             thets
-        }, if (object$method == "weibull-GH") log(object$coefficients$sigma.t) else NULL, 
+        }, if (object$method == "weibull-PH-GH" || object$method == "weibull-AFT-GH") log(object$coefficients$sigma.t) else NULL, 
         if (diag.D) log(D) else chol.transf(D))
     V.thetas <- vcov(object)
     EBs <- ranef(object, postVar = TRUE)
@@ -62,7 +66,7 @@ function (time.points) {
         if (object$method == "ch-GH" || object$method == "ch-Laplace")
             gammas.new[1:nk] <- cumsum(c(gammas.new[1], exp(gammas.new[2:nk])))
         alpha.new <- thetas.new[ncx + ncww + 2]
-        if (object$method == "weibull-GH") {
+        if (object$method == "weibull-PH-GH" || object$method == "weibull-AFT-GH") {
             sigma.t.new <- exp(thetas.new[ncx + ncww + 3])
             D.new <- thetas.new[seq(ncx + ncww + 4, length(thetas))]
             D.new <- if (diag.D) exp(D.new) else chol.transf(D.new)
@@ -120,7 +124,7 @@ function (time.points) {
             fitted.valsM[, ii] <- if (type == "Marginal" || type == "stand-Marginal") {
                 as.vector(X.missM %*% object$coefficients$betas)
             } else {
-                as.vector(X.missM %*% object$coefficients$betas + rowSums(Z.missM * EBs))
+                as.vector(X.missM %*% object$coefficients$betas + rowSums(Z.missM * b.new))
             }
             mu <- as.vector(X.missM %*% betas.new + rowSums(Z.missM * b.new))
             y.new <- rnorm(n, mu, sigma.new)
